@@ -146,32 +146,14 @@ var LibraryBrowser = {
 
       function pointerLockChange() {
         var canvas = Browser.getCanvas();
-        Browser.pointerLock = document['pointerLockElement'] === canvas ||
-                              document['mozPointerLockElement'] === canvas ||
-                              document['webkitPointerLockElement'] === canvas ||
-                              document['msPointerLockElement'] === canvas;
+        Browser.pointerLock = document.pointerLockElement === canvas;
       }
       var canvas = Browser.getCanvas();
       if (canvas) {
         // forced aspect ratio can be enabled by defining 'forcedAspectRatio' on Module
         // Module['forcedAspectRatio'] = 4 / 3;
 
-        canvas.requestPointerLock = canvas['requestPointerLock'] ||
-                                    canvas['mozRequestPointerLock'] ||
-                                    canvas['webkitRequestPointerLock'] ||
-                                    canvas['msRequestPointerLock'] ||
-                                    (() => {});
-        canvas.exitPointerLock = document['exitPointerLock'] ||
-                                 document['mozExitPointerLock'] ||
-                                 document['webkitExitPointerLock'] ||
-                                 document['msExitPointerLock'] ||
-                                 (() => {}); // no-op if function does not exist
-        canvas.exitPointerLock = canvas.exitPointerLock.bind(document);
-
         document.addEventListener('pointerlockchange', pointerLockChange, false);
-        document.addEventListener('mozpointerlockchange', pointerLockChange, false);
-        document.addEventListener('webkitpointerlockchange', pointerLockChange, false);
-        document.addEventListener('mspointerlockchange', pointerLockChange, false);
 
         if (Module['elementPointerLock']) {
           canvas.addEventListener("click", (ev) => {
@@ -754,7 +736,7 @@ var LibraryBrowser = {
 
   // To avoid creating worker parent->child chains, always proxies to execute on the main thread.
   emscripten_create_worker__proxy: 'sync',
-  emscripten_create_worker__deps: ['$UTF8ToString', 'malloc', 'free'],
+  emscripten_create_worker__deps: ['$UTF8ToString', 'realloc'],
   emscripten_create_worker: (url) => {
     url = UTF8ToString(url);
     var id = Browser.workers.length;
@@ -763,7 +745,6 @@ var LibraryBrowser = {
       callbacks: [],
       awaited: 0,
       buffer: 0,
-      bufferSize: 0
     };
     info.worker.onmessage = function info_worker_onmessage(msg) {
       if (ABORT) return;
@@ -781,11 +762,7 @@ var LibraryBrowser = {
       var data = msg.data['data'];
       if (data) {
         if (!data.byteLength) data = new Uint8Array(data);
-        if (!info.buffer || info.bufferSize < data.length) {
-          if (info.buffer) _free(info.buffer);
-          info.bufferSize = data.length;
-          info.buffer = _malloc(data.length);
-        }
+        info.buffer = _realloc(info.buffer, data.length);
         HEAPU8.set(data, info.buffer);
         callbackInfo.func(info.buffer, data.length, callbackInfo.arg);
       } else {
@@ -801,7 +778,7 @@ var LibraryBrowser = {
   emscripten_destroy_worker: (id) => {
     var info = Browser.workers[id];
     info.worker.terminate();
-    if (info.buffer) _free(info.buffer);
+    _free(info.buffer);
     Browser.workers[id] = null;
   },
 
