@@ -162,7 +162,7 @@ var LibraryWget = {
     return handle;
   },
 
-  emscripten_async_wget2_data__deps: ['$wget', 'malloc', 'free'],
+  emscripten_async_wget2_data__deps: ['$wget', 'malloc', 'free', '$stringToUTF8OnStack'],
   emscripten_async_wget2_data__proxy: 'sync',
   emscripten_async_wget2_data: (url, request, param, userdata, free, onload, onerror, onprogress) => {
     var _url = UTF8ToString(url);
@@ -194,7 +194,7 @@ var LibraryWget = {
         var buffer = _malloc(byteArray.length);
         HEAPU8.set(byteArray, buffer);
         if (onload) {{{ makeDynCall('vippi', 'onload') }}}(handle, userdata, buffer, byteArray.length);
-        if (free) _free(buffer);
+        _free(buffer);
       } else {
         onerrorjs();
       }
